using System;
using System.Collections;
using System.Data;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    public class CustomerReviewDetails
    {
        private string _reviewID;
        private string _productID;
        private string _customerID;
        private double _reviewRating;
        private bool _isEnabled;
        private string _subject;
        private string _body;
        private DateTime _reviewDate;
        private string _userName;
        private string _cultureID;

        public string ReviewID { get { return _reviewID; } set { _reviewID = value; } }
        public string ProductID { get { return _productID; } set { _productID = value; } }
        public string CustomerID { get { return _customerID; } set { _customerID = value; } }
        public double ReviewRating { get { return _reviewRating; } set { _reviewRating = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string Subject { get { return _subject; } set { _subject = value; } }
        public string Body { get { return _body; } set { _body = value; } }
        public DateTime ReviewDate { get { return _reviewDate; } set { _reviewDate = value; } }
        public string UserName { get { return _userName; } set { _userName = value; } }
        public string CultureID { get { return _cultureID; } set { _cultureID = value; } }
    }

    public static class CustomerReviewAccess
    {
        private const string AllFields =
            " ReviewID, ProductID, CustomerReview.CustomerID, ReviewRating, Enabled, ReviewDate, Subject, Body, " +
            "UserName, FirstName, LastName, CultureID ";


        private static void SetPrimaryKey( DataTable table )
        {
            DataColumn[] keys = new DataColumn[] { table.Columns["ReviewID"], table.Columns["CultureID"] };
            table.PrimaryKey = keys;
        }

        private static CustomerReviewDetails CopyCustomerReviewDetails( DataRow row )
        {
            CustomerReviewDetails details = new CustomerReviewDetails();

            details.ReviewID = row["ReviewID"].ToString();
            details.ProductID = row["ProductID"].ToString();
            details.CustomerID = row["CustomerID"].ToString();
            details.ReviewRating = Convert.ToDouble( row["ReviewRating"] );
            details.Subject = row["Subject"].ToString();
            details.Body = row["Body"].ToString();
            details.IsEnabled = bool.Parse( row["Enabled"].ToString() );
            details.ReviewDate = Convert.ToDateTime( row["ReviewDate"] );
            details.UserName = row["UserName"].ToString();
            details.CultureID = row["CultureID"].ToString();

            return details;
        }


        public static CustomerReviewDetails GetDetails( string reviewID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM CustomerReview " +
                "LEFT JOIN Customer AS C ON CustomerReview.CustomerID = C.CustomerID " +
                "WHERE ReviewID = @ReviewID  ",
                DataAccess.CreateParameterString( reviewID ) );

            SetPrimaryKey( table );

            if (table.Rows.Count > 0)
                return CopyCustomerReviewDetails( table.Rows[0] );
            else
                return new CustomerReviewDetails();
        }

        public static CustomerReviewDetails GetDetails( string reviewID, string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM CustomerReview " +
                "LEFT JOIN Customer AS C ON CustomerReview.CustomerID = C.CustomerID " +
                "WHERE ReviewID = @ReviewID AND CultureID = @CultureID",
                DataAccess.CreateParameterString( reviewID ),
                DataAccess.CreateParameterString( cultureID ) );

            SetPrimaryKey( table );

            if (table.Rows.Count > 0)
                return CopyCustomerReviewDetails( table.Rows[0] );
            else
                return new CustomerReviewDetails();
        }

        public static double GetAverageRating( string productID )
        {
            DataTable dt = DataAccess.ExecuteSelect(
                "SELECT ReviewRating From CustomerReview WHERE ProductID = @ProductID ",
                DataAccess.CreateParameterString( productID ) );

            int count = 0;

            if (dt.Rows.Count > 0)
            {
                double sum = 0.0;

                foreach (DataRow dr in dt.Rows)
                {
                    if (dr["ReviewRating"].ToString() != "0")
                    {
                        sum += Convert.ToDouble( dr["ReviewRating"] );
                        count += 1;
                    }
                }

                return sum / count;
            }
            else
            {
                return 0.0;
            }
        }

        public static string GetNumberOfRating( string productID )
        {
            string baseOnReview = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM CustomerReview " +
                "WHERE ProductID = @ProductID AND " +
                "    ReviewRating > 0; ",
                DataAccess.CreateParameterString( productID ) );

            return baseOnReview;
        }

        public static string GetProductIDByReviewID( string reviewID )
        {
            string productID = DataAccess.ExecuteScalar(
            "SELECT ProductID From CustomerReview WHERE ReviewID = @ReviewID ",
            DataAccess.CreateParameterString( reviewID ) );

            return productID;
        }

        public static DataTable GetTableSchema()
        {
            string sql =
                "SELECT ReviewID, ProductID, UserName,  ReviewRating, Enabled, ReviewDate, Subject, Body " +
                "FROM CustomerReview " +
                "Left join ( Select * from Customer ) As C on CustomerReview.CustomerID = C.CustomerID  " +
                "Where 1=0; ";

            DataTable table = DataAccess.ExecuteSelect( sql );
            DataColumn[] keys = new DataColumn[] { table.Columns["ReviewID"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static DataTable GetByEnabled( bool enabled, string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ReviewID, ProductID, UserName, FirstName, FirstName + ' ' + LastName As FullName, " +
                "LastName, ReviewRating, Enabled, ReviewDate, Subject, Body, CultureID " +
                "FROM  CustomerReview " +
                "LEFT JOIN  ( Select * from Customer ) As C on CustomerReview.CustomerID = C.CustomerID  " +
                "WHERE Enabled = @Enabled AND ProductID = @ProductID " +
                "ORDER BY ReviewID ",
                DataAccess.CreateParameterBool( enabled ),
                DataAccess.CreateParameterString( productID )
                );

            return table;
        }

        public static DataTable GetByEnabled( bool enabled, string productID, string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ReviewID, ProductID, UserName, FirstName, FirstName + ' ' + LastName As FullName, " +
                "LastName, ReviewRating, Enabled, ReviewDate, Subject, Body, CultureID " +
                "FROM  CustomerReview " +
                "LEFT JOIN  ( Select * from Customer ) As C on CustomerReview.CustomerID = C.CustomerID  " +
                "WHERE Enabled = @Enabled AND ProductID = @ProductID AND CultureID = @CultureID " +
                "ORDER BY ReviewID ",
                DataAccess.CreateParameterBool( enabled ),
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( cultureID )
                );

            return table;
        }

        public static DataTable GetAll()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM CustomerReview " +
                "LEFT JOIN  Customer As C ON CustomerReview.CustomerID = C.CustomerID; "
            );

            SetPrimaryKey( table );

            return table;
        }

        public static DataTable SearchReview(
            string productID,
            string sortBy,
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            string SerialFieldName = fieldName;
            ArrayList parameterList = new ArrayList();
            string commandSearch = SearchFilterHelper.CreateCommandWithWhere( filterType, SerialFieldName, value1, value2, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( productID ) );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT ReviewID, ProductID, UserName, ReviewRating, Enabled, ReviewDate, Subject, Body, CultureID " +
                "FROM CustomerReview  " +
                "LEFT JOIN ( Select * from Customer ) As C on CustomerReview.CustomerID = C.CustomerID  " +
                commandSearch +
                " And ProductID = @ProductID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ReviewID"], full.Columns["CultureID"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public static DataTable SearchReview(
            string productID,
            string cultureID,
            string sortBy,
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            string SerialFieldName = fieldName;
            ArrayList parameterList = new ArrayList();
            string commandSearch = SearchFilterHelper.CreateCommandWithWhere( filterType, SerialFieldName, value1, value2, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( productID ) );
            parameterList.Add( DataAccess.CreateParameterString( cultureID ) );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT ReviewID, ProductID, UserName, ReviewRating, Enabled, ReviewDate, Subject, Body, CultureID " +
                "FROM CustomerReview  " +
                "LEFT JOIN ( Select * from Customer ) As C on CustomerReview.CustomerID = C.CustomerID  " +
                commandSearch +
                " And ProductID = @ProductID AND CultureID = @CultureID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ReviewID"], full.Columns["CultureID"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "reviewid" ))
                return "ORDER BY CustomerReview." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        public static string Create(
            string productID,
            string cultureID,
            string customerID,
            double reviewRating,
            bool enabled,
            string subject,
            string body )
        {
            string ReviewID = DataAccess.ExecuteScalar(
                "INSERT INTO CustomerReview (ProductID, CustomerID, ReviewRating, Enabled, Subject, Body, CultureID ) " +
                "VALUES ( @productID, @customerID, @reviewRating, @enabled, @subject, @body, @cultureID ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterDouble( reviewRating ),
                DataAccess.CreateParameterBool( enabled ),
                DataAccess.CreateParameterString( subject ),
                DataAccess.CreateParameterString( body ),
                DataAccess.CreateParameterString( cultureID )
                );


            return ReviewID;
        }

        public static void Update(
            string reviewID,
            string productID,
            string cultureID,
            string customerID,
            double reviewRating,
            DateTime reviewDate,
            bool enabled,
            string subject,
            string body )
        {

            DataAccess.ExecuteNonQuery(
                "UPDATE CustomerReview " +
                "SET ProductID = @ProductID, " +
                "    CustomerID = @CustomerID, " +
                "    ReviewRating = @ReviewRating, " +
                "    Enabled = @Enabled, " +
                "    ReviewDate = @ReviewDate, " +
                "    Subject = @Subject, " +
                "    Body = @Body " +
                "WHERE ReviewID = @ReviewID AND CultureID = @CultureID",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterDouble( reviewRating ),
                DataAccess.CreateParameterBool( enabled ),
                DataAccess.CreateParameterDateTime( reviewDate ),
                DataAccess.CreateParameterString( subject ),
                DataAccess.CreateParameterString( body ),
                DataAccess.CreateParameterString( reviewID ),
                DataAccess.CreateParameterString( cultureID ) );

        }

        public static void Delete( string reviewID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM CustomerReview " +
                "WHERE ReviewID = @ReviewID; ",

                DataAccess.CreateParameterString( reviewID ) );
        }

        public static void DeleteAll()
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM CustomerReview " );
        }

    }
}
