using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Text;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.DataAccessLib
{
    public class UrlNameAccessHelper
    {
        private const int PrimaryKeyIndex = 0;
        private const int NameIndex = 1;

        private const int RandomUrlDigitsCount = 7;

        private string _tableName;
        private string _primaryKey;


        public UrlNameAccessHelper( string tableName, string primaryKey )
        {
            _tableName = tableName;
            _primaryKey = primaryKey;
        }


        private bool IsUrlNameEmpty( string primaryKeyID )
        {
            string urlName = DataAccess.ExecuteScalar(
                "SELECT UrlName " +
                "FROM " + _tableName + " " +
                "WHERE " + _primaryKey + " = @PrimaryKeyID; ",
                DataAccess.CreateParameterString( primaryKeyID ) );

            return String.IsNullOrEmpty( urlName );
        }

        private bool IsUrlNameExisting( DataTable table, string urlName )
        {
            foreach (DataRow row in table.Rows)
            {
                if (String.Equals( row["UrlName"].ToString(), urlName,
                    StringComparison.OrdinalIgnoreCase ))
                {
                    return true;
                }
            }

            return false;
        }

        private bool IsUrlNameExisting( IDictionary existingUrlNames, string urlName )
        {
            return existingUrlNames.Contains( urlName.ToLower() );
        }

        private void AddNewUrlName( IDictionary existingUrlNames, string newName )
        {
            existingUrlNames[newName.ToLower()] = "";
        }

        private string CreateBaseUrlName( string name )
        {
            string normalized = name.Normalize( NormalizationForm.FormC );

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < normalized.Length; i++)
            {
                if (StringUtilities.IsUnicodeLetterOrDigit( normalized[i] ))
                    sb.Append( normalized[i] );
                else
                    sb.Append( '_' );
            }

            return sb.ToString().Trim( new char[] { ' ', '_' } );
        }

        private string ConvertToUrlName( string primaryKeyID, string name, bool isLocale )
        {
            string baseUrlName = CreateBaseUrlName( name );

            string sql = "SELECT UrlName " +
                "FROM " + _tableName + " " +
                "WHERE UrlName LIKE @UrlName AND " +
                _primaryKey + " <> @PrimaryKeyID; ";

            DataTable existingNames = DataAccess.ExecuteSelect(
                sql,   // Allow convert to the same URL name as itself
                DataAccess.CreateParameterString( baseUrlName + "%" ),
                DataAccess.CreateParameterString( primaryKeyID ) );

            string urlName = baseUrlName;
            if (IsUrlNameExisting( existingNames, urlName ))
            {
                int i = 1;
                do
                {
                    urlName = baseUrlName + '-' + ConvertToCharacterIndex( i );
                    i++;
                } while (IsUrlNameExisting( existingNames, urlName ));
            }

            return urlName;
        }

        private string ConvertToUrlName( IDictionary existingUrlNames, string name )
        {
            string baseUrlName = CreateBaseUrlName( name );

            string urlName = baseUrlName;
            if (IsUrlNameExisting( existingUrlNames, urlName ))
            {
                int i = 1;
                do
                {
                    urlName = baseUrlName + '-' + ConvertToCharacterIndex( i );
                    i++;
                } while (IsUrlNameExisting( existingUrlNames, urlName ));
            }

            return urlName;
        }

        // Convert an integer index to a character index, using a = 0, b = 1, c = 2, etc.
        // If the index exceeds 25, more than one character will be used in the index, 
        // e.g. bc = (1 * 26) + 2 = 28
        private static string ConvertToCharacterIndex( int index )
        {
            string result = string.Empty;
            int number = index;

            while (number != 0)
            {
                int remaining;
                number = Math.DivRem( number, 26, out remaining );
                result = Convert.ToChar( 'a' + remaining ) + result;
            }

            return result;
        }

        private void UpdateDatabase( string urlName, string primaryKeyID )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE " + _tableName + " " +
                "SET UrlName = @UrlName " +
                "WHERE " + _primaryKey + " = @PrimaryKeyID; ",
                DataAccess.CreateParameterString( urlName ),
                DataAccess.CreateParameterString( primaryKeyID ) );
        }

        private DataTable GetHostDataAll( string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _primaryKey + ", Name " +
                "FROM " + _tableName + "Locale " +
                "WHERE CultureID = @CultureID AND Name = @Name; ",
                DataAccess.CreateParameterString( DataAccessHelper.UrlCultureID ),
                DataAccess.CreateParameterString( name ) );

            return table;
        }

        private DataTable GetHostDataAll()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _primaryKey + ", Name " +
                "FROM " + _tableName + "Locale " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( DataAccessHelper.UrlCultureID ) );

            return table;
        }

        public string CreateLocaleUrlName( string name )
        {
            return ConvertToUrlName( String.Empty, name, true );
        }


        public string UpdateUrlName(
            string primaryKeyID,
            string name )
        {
            string urlName = ConvertToUrlName( primaryKeyID, name, false );
            UpdateDatabase( urlName, primaryKeyID );

            return urlName;
        }

        public string CreateUrlName(
            string cultureID,
            string primaryKeyID,
            string name )
        {
            if (cultureID == DataAccessHelper.UrlCultureID)
            {
                return UpdateUrlName( primaryKeyID, name );
            }
            else
            {
                return UpdateUrlName( primaryKeyID, RandomUtilities.RandomNumberNDigits( RandomUrlDigitsCount ) );
            }
        }

        public string UpdateUrlNameConditionally(
            string cultureID,
            string primaryKeyID,
            string name )
        {
            if (cultureID == DataAccessHelper.UrlCultureID)
            {
                return UpdateUrlName( primaryKeyID, name );
            }
            else
                return null;
        }

        public string CreateNoCultureUrlName(
            string primaryKeyID,
            string name )
        {
            return UpdateUrlName( primaryKeyID, name );
        }
        public string UpdateNoCultureUrlNameConditionally(
            string primaryKeyID,
            string name )
        {
            return UpdateUrlName( primaryKeyID, name );
        }
        public void UpdateUrlNameAll( string name )
        {
            Hashtable existingUrlNames = new Hashtable();

            DataTable hostTable = GetHostDataAll( name );

            foreach (DataRow row in hostTable.Rows)
            {
                string urlName = ConvertToUrlName( existingUrlNames, row[NameIndex].ToString() );
                UpdateDatabase( urlName, row[PrimaryKeyIndex].ToString() );

                AddNewUrlName( existingUrlNames, urlName );
            }
        }

        public void UpdateUrlNameAll()
        {
            Hashtable existingUrlNames = new Hashtable();

            DataTable hostTable = GetHostDataAll();

            foreach (DataRow row in hostTable.Rows)
            {
                string urlName = ConvertToUrlName( existingUrlNames, row[NameIndex].ToString() );
                UpdateDatabase( urlName, row[PrimaryKeyIndex].ToString() );

                AddNewUrlName( existingUrlNames, urlName );
            }
        }

    }
}
