using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Stores;
using System.Web.Caching;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ConfigurationRepository : IConfigurationRepository
    {
        #region Private

        private const string RepositoryName = "ConfigurationRepository";
        private const string GetAllSignature = "GetAll";
        private const string SearchConfigurationSignature = "SearchConfiguration";
        private const string GetByParentConfigNameSignature = "GetByParentConfigName";
        private const string GetByEnablerConfigNameSignature = "GetByEnablerConfigName";

        private DataCache _dataCache = new DataCache( RepositoryName );

        private ConfigurationRepositoryNoCache _repository = new ConfigurationRepositoryNoCache();
        private ICacheDependencyUtility _cacheDependency 
            = new CacheDependencyUtility( new string[] { "Configuration", "ConfigurationValue", "ConfigurationDescription" } );

        #endregion

        #region Constructors
        public ConfigurationRepository()
        {
        }

        public ConfigurationRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }
        #endregion

        #region Public Methods

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _dataCache.ClearRepositoryCache();
        }

        public Configuration Save( Configuration configuration )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Configuration newConfiguration = _repository.Save( configuration );
            _dataCache.ClearRepositoryCache();
            return newConfiguration;
        }

        public Configuration UpdateValue( Configuration configuration, string newValue )
        {
            Configuration result = _repository.UpdateValue( configuration, newValue );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Configuration UpdateValue( Configuration configuration, string newValue, Store store )
        {
            Configuration result = _repository.UpdateValue( configuration, newValue, store );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Configuration UpdateValue( Culture culture, Configuration configuration, string newValue )
        {
            Configuration result = _repository.UpdateValue( culture, configuration, newValue );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Configuration UpdateValue( Culture culture, Configuration configuration, string newValue, Store store )
        {
            Configuration result = _repository.UpdateValue( culture, configuration, newValue, store );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public IList<Configuration> GetAll()
        {
            IList<Configuration> list = (IList<Configuration>) _dataCache.GetValue( GetAllSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetAll();

                _dataCache.SetValue( GetAllSignature, list );
            }

            return list;
        }

        public IList<Configuration> SearchConfiguration(
            Culture culture, string sortBy, string searchTerm, string[] searchFieldList )
        {
            Pair result = (Pair) _dataCache.GetValue(
                SearchConfigurationSignature,
                _cacheDependency,
                sortBy,
                searchTerm,
                string.Join(",", searchFieldList)
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchConfiguration(
                    culture, sortBy, searchTerm, searchFieldList );
                _dataCache.SetValue(
                    SearchConfigurationSignature,
                    result,
                    sortBy,
                    searchTerm,
                    string.Join( ",", searchFieldList )
                    );
            }

            return (IList<Configuration>) result.First;
        }

        public IList<Configuration> GetByParentConfigName( string configName )
        {
            IList<Configuration> list = (IList<Configuration>) _dataCache.GetValue(
                GetByParentConfigNameSignature, _cacheDependency, configName );

            if (list == null)
            {
                list = _repository.GetByParentConfigName( configName );
                _dataCache.SetValue( GetByParentConfigNameSignature, list, configName );
            }

            return list;
        }

        public IList<Configuration> GetByEnablerConfigName( string configName )
        {
            IList<Configuration> list = (IList<Configuration>) _dataCache.GetValue(
                GetByEnablerConfigNameSignature, _cacheDependency, configName );

            if (list == null)
            {
                list = _repository.GetByEnablerConfigName( configName );
                _dataCache.SetValue( GetByEnablerConfigNameSignature, list, configName );
            }

            return list;
        }

        public void DeleteConfigValueByStoreID( string storeID )
        {
            _repository.DeleteConfigValueByStoreID( storeID );
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
