using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Shared.DataAccess;
using Vevo.Domain.Base;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Data.Implementation;
using Vevo.Base.Domain;

namespace Vevo.Data.Implementation
{
    internal class ProductImageAccess : BaseListedDataAccess<string, ProductImage>
    {
        #region Private

        private const string AllProductImageFields = " ProductImageID, ProductID, ThumbnailImage, RegularImage, LargeImage, " +
            "SortOrder, IsZoom, IsEnlarge ";

        private const string AllProductImageLocaleFields = " ProductImageID, CultureID, AltTag, TitleTag ";

        private ProductImage CreateOneFromRow( DataRow row )
        {
            ProductImage productImage = new ProductImage();
            productImage.ProductImageID = ConvertUtilities.ToString( row["ProductImageID"] );
            productImage.ThumbnailImage = ConvertUtilities.ToString( row["ThumbnailImage"] );
            productImage.RegularImage = ConvertUtilities.ToString( row["RegularImage"] );
            productImage.LargeImage = ConvertUtilities.ToString( row["LargeImage"] );
            productImage.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            productImage.IsZoom = ConvertUtilities.ToBoolean( row["IsZoom"] );
            productImage.IsEnlarge = ConvertUtilities.ToBoolean( row["IsEnlarge"] );

            //details.ProductID = table.Rows[0]["ProductID"].ToString();
            return productImage;
        }

        private ProductImageLocale CreateOneLocaleFromRow( DataRow row)
        {
            ProductImageLocale locale = new ProductImageLocale();
            locale.CultureID = ConvertUtilities.ToString( row["CultureID"] );
            locale.AltTag = ConvertUtilities.ToString( row["AltTag"] );
            locale.TitleTag = ConvertUtilities.ToString( row["TitleTag"] );
            return locale;
        }

        private IEntityList<ProductImage> ConvertTableToList( DataTable table )
        {
            IEntityList<ProductImage> list = new EntityList<ProductImage>();
            foreach (DataRow row in table.Rows)
            {
                ProductImage image = CreateOneFromRow( row);
                IEntityList<ProductImageLocale> imageLocales = GetImageLocale( image.ProductImageID );
                foreach (ProductImageLocale locale in imageLocales)
                {
                    image.Locales.Add( locale );
                }
                list.Add( CreateOneFromRow( row ) );
            }
            return list;
        }

        private IEntityList<ProductImageLocale> ConvertLocaleTableToList( DataTable table )
        {
            IEntityList<ProductImageLocale> list = new EntityList<ProductImageLocale>();
            foreach (DataRow row in table.Rows)
            {
                ProductImageLocale imageLocale = CreateOneLocaleFromRow( row);
                list.Add( imageLocale );
            }
            return list;
        }

        #endregion

        #region ProductImageLocale

        private void CreateProductImageLoale( string productImageID, ProductImageLocale locale )
        {
            DataAccess.ExecuteScalar(
                   "INSERT INTO ProductImageLocale (ProductImageID, CultureID, AltTag, TitleTag) " +
                    "VALUES (@productImageID, @CultureID, @AltTag, @Title);",
                    DataAccess.CreateParameterString( productImageID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.AltTag ),
                    DataAccess.CreateParameterString( locale.TitleTag )
            );
        }

        private void DeleteProductImageLocale( string productImageID)
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductImageLocale " +
                "WHERE ProductImageID = @ProductImageID;",
                DataAccess.CreateParameterString( productImageID )
            );
        }

        #endregion

        #region Protected

        protected override void Create( string productID, ProductImage item )
        {
            string productImageID = DataAccess.ExecuteScalar(
                "INSERT INTO ProductImage (ProductID, RegularImage, LargeImage, ThumbnailImage, SortOrder, " +
                "IsZoom, IsEnlarge) " +
                "VALUES (@productID, @RegularImage, @LargeImage, @ThumbnailImage, @SortOrder, @IsZoom, @IsEnlarge);" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( item.RegularImage ),
                DataAccess.CreateParameterString( item.LargeImage ),
                DataAccess.CreateParameterString( item.ThumbnailImage ),
                DataAccess.CreateParameterInt32( item.SortOrder ),
                DataAccess.CreateParameterBool( item.IsZoom ),
                DataAccess.CreateParameterBool( item.IsEnlarge )
                );
        }

        protected override void Update( string parentID, ProductImage item )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ProductImage " +
                "SET ThumbnailImage = @ThumbnailImage, " +
                " RegularImage = @RegularImage, " +
                " LargeImage = @LargeImage, " +
                " SortOrder = @SortOrder, " +
                " IsZoom = @IsZoom, " +
                " IsEnlarge = @IsEnlarge " +
                " WHERE ProductImageID = @ProductImageID;",
                DataAccess.CreateParameterString( item.ThumbnailImage ),
                DataAccess.CreateParameterString( item.RegularImage ),
                DataAccess.CreateParameterString( item.LargeImage ),
                DataAccess.CreateParameterInt32( item.SortOrder ),
                DataAccess.CreateParameterBool( item.IsZoom ),
                DataAccess.CreateParameterBool( item.IsEnlarge ),
                DataAccess.CreateParameterString( item.ProductImageID ) );

            DeleteProductImageLocale( item.ProductImageID );
            foreach (ProductImageLocale locale in item.Locales)
            {
                CreateProductImageLoale( item.ProductImageID, locale );
            }
        }

        protected override void Delete( string parentID, ProductImage item )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductImage " +
                "WHERE ProductImageID = @ProductImageID;",
                DataAccess.CreateParameterString( item.ProductImageID )
                );

            DeleteProductImageLocale( item.ProductImageID );
        }

        #endregion

        #region Public Methods

        public void DeleteByProductID( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductImage " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );
        }

        public IEntityList<ProductImage> GetByProductID( string productID )
        {
            DataTable table = new DataTable();
            table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageFields +
                "FROM ProductImage " +
                "WHERE ProductID = @ProductID " +
                "ORDER BY SortOrder;",
                DataAccess.CreateParameterString( productID )
                );

            return ConvertTableToList( table );
        }

        public IEntityList<ProductImageLocale> GetImageLocale( string productImageID )
        {
            DataTable table = new DataTable();
            table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageLocaleFields +
                "FROM ProductImageLocale " +
                "WHERE ProductImageID = @ProductImageID ;",
                DataAccess.CreateParameterString( productImageID )
                );

            return ConvertLocaleTableToList( table );
        }

        public ProductImageLocale GetImageLocale( string productImageID,string cultureID )
        {
            DataTable table = new DataTable();
            table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageLocaleFields +
                "FROM ProductImageLocale " +
                "WHERE ProductImageID = @ProductImageID AND CultureID = @CultureID;",
                DataAccess.CreateParameterString( productImageID ),
                DataAccess.CreateParameterString( cultureID )
                );

            IEntityList<ProductImageLocale> locales = ConvertLocaleTableToList( table );
            if (locales.Count > 0)
                return locales[0];
            else
                return null;
        }

        public void DeletePrimaryImage( string productID, IEntityList<ProductImage> productImages )
        {
            foreach (ProductImage productImage in productImages)
            {
                if (productImage.SortOrder == 0)
                    Delete( productID, productImage );
                else
                {
                    productImage.SortOrder = productImage.SortOrder - 1;
                    Update( productID, productImage );
                }
            }
        }

        public bool ImageIsUsedByAnotherProduct( string largeFilePath )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageFields +
                "FROM ProductImage " +
                "WHERE LargeImage = @LargeImage; ",
                DataAccess.CreateParameterString( largeFilePath ) );

            DataColumn[] keys = new DataColumn[] { table.Columns["ProductImageID"] };
            table.PrimaryKey = keys;
            return table.Rows.Count > 0;
        }

        public bool IsRegularImageExisting( string regularImagePath )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageFields +
                "FROM ProductImage " +
                "WHERE RegularImage = @RegularImage",
                DataAccess.CreateParameterString( regularImagePath ) );

            if (table.Rows.Count > 1)
                return true;
            else
                return false;
        }

        public bool IsLargeImageExisting( string largeImagePath )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageFields +
                "FROM ProductImage " +
                "WHERE LargeImage = @LargeImage",
                DataAccess.CreateParameterString( largeImagePath ) );

            if (table.Rows.Count > 1)
                return true;
            else
                return false;
        }

        public bool IsThumbnailImageExisting( string thumbnailImagePath )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductImageFields +
                "FROM ProductImage " +
                "WHERE ThumbnailImage = @ThumbnailImage",
                DataAccess.CreateParameterString( thumbnailImagePath ) );

            if (table.Rows.Count > 1)
                return true;
            else
                return false;
        }

        #endregion
    }
}
