using System;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;
using Vevo.Domain;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Products.BackOffice
{
    /// <summary>
    /// Summary description for ProductImageFile
    /// </summary>
    public class ProductImageFile : IDisposable
    {
        #region Private

        private const int ThumbnailImageSize = 70;
        private const int KiloBytesValue = 1024;

        private IFileManager _fileManager;
        private string _fileName;
        private Image _largeImage;


        private static ImageFormat GetImageFormat( string fileExtension )
        {
            string extensionWithDot = fileExtension.StartsWith( "." ) ? fileExtension : "." + fileExtension;

            ImageFormat result = null;
            switch (extensionWithDot.ToLower())
            {
                case ".bmp":
                    result = ImageFormat.Bmp;
                    break;
                case ".gif":
                    result = ImageFormat.Gif;
                    break;
                case ".png":
                    result = ImageFormat.Png;
                    break;
                case ".tif":
                    result = ImageFormat.Tiff;
                    break;
                case ".jpg":
                case ".jpeg":
                    result = ImageFormat.Jpeg;
                    break;
            }

            return result;
        }

        private static bool ThumbnailCallback()
        {
            return false;
        }

        private static string GetPathWithTilde( string path )
        {
            if (path.Trim().StartsWith( "~/" ))
                return path.Trim();
            else
                return "~/" + path.Trim();
        }

        private string GetLocalPath( string path )
        {
            return _fileManager.GetFullPath( GetPathWithTilde( path ) );
        }

        private int GetImageHeight( int newImageWidth )
        {
            return (_largeImage.Height * newImageWidth) / _largeImage.Width;
        }

        private string AppendFileName( string fileName, string suffix )
        {
            return Path.GetFileNameWithoutExtension( fileName ) + suffix + Path.GetExtension( fileName );
        }

        private void Initialize()
        {
            if (_fileManager.Exists( GetPathWithTilde( LargeFilePath ) ))
            {
                _largeImage = Image.FromFile( GetLocalPath( LargeFilePath ) );
            }
            else if (_fileManager.Exists( GetPathWithTilde( RegularFilePath ) ))
            {
                _largeImage = Image.FromFile( GetLocalPath( RegularFilePath ) );
            }
        }

        private void ScaleImageSize( int maxWidth, int maxHeight, out int finalWidth, out int finalHeight )
        {
            int nWidth, nHeight;

            if (_largeImage.Width > maxWidth)
            {
                nWidth = maxWidth;
                nHeight = (nWidth * _largeImage.Height) / _largeImage.Width;
            }
            else
            {
                nWidth = _largeImage.Width;
                nHeight = _largeImage.Height;
            }

            if (nHeight > maxHeight)
            {
                finalHeight = maxHeight;
                finalWidth = (nWidth * finalHeight) / nHeight;
            }
            else
            {
                finalWidth = nWidth;
                finalHeight = nHeight;
            }
        }

        private void SaveSmallImage( string pathFileSave, int maxWidth, int maxHeight )
        {
            if (!_fileManager.Exists( GetPathWithTilde( LargeFilePath ) ))
            {
                return;
            }

            using (Bitmap thumbNailImg = new Bitmap( maxWidth, maxHeight, PixelFormat.Format32bppArgb ))
            {
                using (Graphics new_g = Graphics.FromImage( thumbNailImg ))
                {
                    new_g.SmoothingMode = SmoothingMode.AntiAlias;
                    new_g.InterpolationMode = InterpolationMode.HighQualityBicubic;

                    //for .gif with white bg//
                    Rectangle rectangle = new Rectangle( 0, 0, maxWidth, maxHeight );
                    SolidBrush sb = new SolidBrush( System.Drawing.Color.White );

                    new_g.DrawImage( _largeImage, rectangle );

                    if (GetImageFormat( Path.GetExtension( LargeFilePath ) ) == ImageFormat.Gif)
                        thumbNailImg.Save( GetLocalPath( pathFileSave ), ImageFormat.Png );
                    else
                        thumbNailImg.Save( GetLocalPath( pathFileSave ), GetImageFormat( Path.GetExtension( LargeFilePath ) ) );

                    thumbNailImg.Dispose();
                    new_g.Dispose();
                }
            }
        }

        private static bool IsImagePathInLargeDirectory( IFileManager fileManager, string imagePath )
        {
            string localImagePath = fileManager.GetFullPath( GetPathWithTilde( imagePath ) );
            string localLargeDirectory = fileManager.GetFullPath( GetPathWithTilde( LargeRelativeDirectory ) );

            return localImagePath.Contains( localLargeDirectory );
        }

         #endregion


        #region Public Constances

        public const string ThumbnailRelativeDirectory = "Images/Products/Thumbnail/";
        public const string RegularRelativeDirectory = "Images/Products/";
        public const string LargeRelativeDirectory = "Images/Products/Large/";

        #endregion


        #region Constructors

        // Private constructor
        private ProductImageFile( IFileManager fileManager, string fileName )
        {
            _fileManager = fileManager;
            _fileName = fileName;
        }

        public ProductImageFile( IFileManager fileManager, string fileName, Image largeImage )
        {
            _fileManager = fileManager;
            _fileName = fileName;
            _largeImage = largeImage;
        }

        #endregion


        #region Public Propterties

        public string SecondaryFilePath
        {
            get
            {
                return RegularRelativeDirectory + AppendFileName( _fileName, "_s" );
            }
        }

        public string ThumbnailFilePath
        {
            get
            {
                return ThumbnailRelativeDirectory + _fileName;
            }
        }

        public string RegularFilePath
        {
            get
            {
                return RegularRelativeDirectory + _fileName;
            }
        }

        public string LargeFilePath
        {
            get
            {
                return LargeRelativeDirectory + _fileName;
            }
        }

        public int LargeImageWidth
        {
            get
            {
                return _largeImage.Width;
            }
        }

        public int LargeImageHeight
        {
            get
            {
                return _largeImage.Height;
            }
        }

        #endregion


        #region Public Methods

        public static bool IsImageInLargeDirectory( IFileManager fileManager, string imageFilename )
        {
            string localLargeDirectory = GetPathWithTilde( LargeRelativeDirectory );
            return fileManager.Exists( localLargeDirectory + imageFilename );
        }

        public static bool IsImageInRegularDirectory( IFileManager fileManager, string imageFilename )
        {
            string localRegularDirectory = GetPathWithTilde( RegularRelativeDirectory  );
            return fileManager.Exists( localRegularDirectory + imageFilename );
        }

        public static bool IsImageInThumbnailDirectory( IFileManager fileManager, string imageFilename )
        {
            string localThumbnailDirectory = GetPathWithTilde( ThumbnailRelativeDirectory );
            return fileManager.Exists( localThumbnailDirectory + imageFilename );
        }

        public static ProductImageFile Load( IFileManager fileManager, string fileName )
        {
            ProductImageFile imageFile = new ProductImageFile( fileManager, fileName );
            imageFile.Initialize();
            return imageFile;
        }

        public static bool IsValidFileType( string fileName )
        {
            string fileExtension = Path.GetExtension( fileName );
            return GetImageFormat( fileExtension ) != null;
        }

        public static int GetImageSize( IFileManager fileManager, string relativeFilePath )
        {
            if (!fileManager.Exists( GetPathWithTilde( relativeFilePath ) ))
                return 0;
            else
                return (int) fileManager.GetFileSize( GetPathWithTilde( relativeFilePath ) ) / KiloBytesValue;
        }

        public static string CopyToLargeImageFolder( IFileManager fileManager, string imagePath )
        {
            if (!IsImagePathInLargeDirectory( fileManager, imagePath ))
            {
                string destinationPath = GetPathWithTilde( LargeRelativeDirectory + Path.GetFileName( imagePath ) );
                if (fileManager.Exists( destinationPath ))
                {
                    fileManager.Delete( destinationPath );
                }
                fileManager.Copy( GetPathWithTilde( imagePath ), destinationPath );
            }

            return LargeRelativeDirectory + Path.GetFileName( imagePath );
        }

        public void Dispose()
        {
            if (_largeImage != null)
                _largeImage.Dispose();
        }

        public void SaveLargeImage()
        {
            _largeImage.Save( GetLocalPath( LargeFilePath ) );
        }

        public void SaveRegular()
        {
            int regularWidth = DataAccessContext.Configurations.GetIntValue( "RegularImageWidth" );
            int regularHeight = GetImageHeight( regularWidth );

            int finalWidth, finalHeight;
            ScaleImageSize( regularWidth, regularHeight, out finalWidth, out finalHeight );

            SaveSmallImage( RegularFilePath, finalWidth, finalHeight );
        }

        public void SaveThumbnail()
        {
            SaveSmallImage( ThumbnailFilePath, ThumbnailImageSize, ThumbnailImageSize );
        }

        public void SaveSecondary()
        {
            int secondaryWidth = DataAccessContext.Configurations.GetIntValue( "SecondaryImageWidth" );
            int secondaryHeight = GetImageHeight( secondaryWidth );

            int finalWidth, finalHeight;
            ScaleImageSize( secondaryWidth, secondaryHeight, out finalWidth, out finalHeight );
            SaveSmallImage( SecondaryFilePath, finalWidth, finalHeight );
        }

        #endregion

    }
}
